############################################################################
#    Copyright (C) 2007 by the Gang of 6 Pyweek4 Team			   #
#									   #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

import pygame
from pygame.rect import Rect
from pygame.locals import *
import math
import random
from constants import *
import castletiles


# the following line is not needed if pgu is installed
import sys; sys.path.insert(0, "..")

from pgu import tilevid, timer
import hud

SPEED = 2

PLAYER_POS = 0,0
MAX_LINEAR_ENEMY_BOUNDARY = 20

def load_game():
	data = {} # dictinory holds the settings
	if os.path.exists(SAVE_GAME_PATH) == False:
		# set the defaults
		for defEntry in SAVE_GAME_KEYS.items():
			data[defEntry[0]] = defEntry[1]
		
	if os.path.isfile(SAVE_GAME_PATH):
		f = file(SAVE_GAME_PATH, 'r')
		
		lines = f.readlines()
		for line in lines:
			strings = line.split()
			if(len(strings) != 2):
				print "Invalid game data!\nPlease delete .midnight!"
				#sys.exit()
			
			data[strings[0]] = strings[1]
		
		f.close()

	return data

def save_game(data):
		print data
		f = file(SAVE_GAME_PATH, 'w')
		for opt in data:
				f.write(opt + '\t' + str(data[opt]) + '\n')
		f.close()
		
	
def has_valid_save_game():
		try:
				f = file(SAVE_GAME_PATH, 'r')
		except IOError:
				return False
				
		# ok we have the file, now see if it's valid or not
		data = load_game()
		for key in SAVE_GAME_KEYS.keys():
			# it's invalid if it's missing even one entry
			if not key in data:
				return False
				
		return True

def load_game_nvm(data,key,default):
	if key in data:
		if type(default) == bool:
			return makeBool(data[key])
		else:
			return type(default)(data[key])

	return default





def player_new(g,t,value):
	g.clayer[t.ty][t.tx] = 0
	s = tilevid.Sprite(g.images['player'],t.rect)
	g.sprites.append(s)
	s.loop = player_loop
	s.groups = g.string2groups('player')
	s.score = 0
	g.player = s
	##In player_new() I add the shoot handler.
	##::
	s.shoot = player_shoot
    ##

def player_loop(g,s):
	if g.TURN == True:
		#if s.rect.right < g.view.left: g.quit = 1
		
		#s.rect.x += SPEED
		
		keys = pygame.key.get_pressed()
		dx,dy = 0,0
		if keys[K_UP] or keys[K_KP8]: 
			dy-=2
			g.TURN = False
			#g.view.y -= 10
			
	
		if keys[K_DOWN] or keys[K_KP2]: 
			dy+=2
			g.TURN = False
			#g.view.y += 10
	
		if keys[K_LEFT] or keys[K_KP4]: 
			dx-=2
			g.TURN = False
			#g.view.x -= 10
	
		if keys[K_RIGHT] or keys[K_KP6]: 
			dx+=2
			g.TURN = False
			#g.view.x += 10
	
		s.rect.x += dx*5
		s.rect.y += dy*5
		PLAYER_POS = s.rect.x, s.rect.y
		s.rect.clamp_ip(g.view)
		
		
		##In player_loop(), I check for the spacebar.  The spacebar triggers a shot every 8 frames.
		##::
		if keys[K_SPACE] and g.frame%7==0:
			if g.TURN == True:
				shot_new(g,s,None)
				#g.view.x += 10
				g.TURN = False
				print PLAYER_POS
	else:
		keys = pygame.key.get_pressed()
		if keys[K_DELETE]:
			g.TURN = True
	g.TURN = True
##The player_shoot() handler, as well as the shot Sprite functions.  The shot group
##has its agroup set to 'enemy' so it can hit 'enemy' Sprites.
##::
def player_shoot(g,s):
	shot_new(g,s,None)
    
def shot_new(g,t,value):
	s = tilevid.Sprite(g.images['shot'],(t.rect.right,t.rect.centery-2))
	g.sprites.append(s)
	s.agroups = g.string2groups('enemy')
	s.hit = shot_hit
	s.loop = shot_loop

def shot_loop(g,s):
	s.rect.x += 8
	if s.rect.left > g.view.right:
		g.sprites.remove(s)
##    
        
def shot_hit(g,s,a):
	if a in g.sprites: g.sprites.remove(a)
	g.player.score += 500

        
def enemy_new(g,t,value):
	g.clayer[t.ty][t.tx] = 0
	s = tilevid.Sprite(g.images['enemy'],t.rect)
	g.sprites.append(s)
	s.loop = enemy_loop
##In enemy_new(), I've added a lot more detail.
##- A move function to handle the type of movement the enemy will do.
##- A record of the origin and entering frame of the enemy (useful for the move functions.)
##- Set up the groups and agroups and a hit handler for the enemy.
##::
	s.move = value['move']
	s.origin = pygame.Rect(s.rect)
	s.direction = -1
	s.frame = g.frame
	s.groups = g.string2groups('enemy')
	s.agroups = g.string2groups('player')
	s.hit = enemy_hit
##

##When an enemy is hit, the game quits.
##::
def enemy_hit(g,s,a):
	g.quit = 1
##

def enemy_loop(g,s):	
	##In enemy_loop() we call the move handler.
	##::
	s.move(g,s)

	
	if s.rect.left < g.view.left:
		#g.sprites.remove(s)
		s.direction = 1

##The enemy movement handlers.
##::
def enemy_move_line(g,s):			
	if s.rect.x < s.origin.x - MAX_LINEAR_ENEMY_BOUNDARY:
		s.direction = 1
	elif s.rect.x > s.origin.x + MAX_LINEAR_ENEMY_BOUNDARY:
		s.direction = -1
	s.rect.x += s.direction
    
def enemy_move_sine(g,s):
	s.rect.x -= 2
	s.rect.y = s.origin.y + 65*math.sin((g.frame-s.frame)/10.0)
    
def enemy_move_circle(g,s):
	s.origin.x -= 1
	s.rect.y = s.origin.y + 50*math.sin((g.frame-s.frame)/10.0)
	s.rect.x = s.origin.x + 50*math.cos((g.frame-s.frame)/10.0)
##
        
def cobble_road_block(g,t,a):
	c = t.config
	if (c['top'] == 1 and a._rect.bottom <= t._rect.top and a.rect.bottom > t.rect.top):
		a.rect.bottom = t.rect.top
	if (c['left'] == 1 and a._rect.right <= t._rect.left and a.rect.right > t.rect.left):
		a.rect.right = t.rect.left
	if (c['right'] == 1 and a._rect.left >= t._rect.right and a.rect.left < t.rect.right):
		a.rect.left = t.rect.right
	if (c['bottom'] == 1 and a._rect.top >= t._rect.bottom and a.rect.top < t.rect.bottom):
		a.rect.top = t.rect.bottom

def tile_coin(g,t,a):
	a.score += 100
	g.set((t.tx,t.ty),0)
    
def tile_fire(g,t,a):
	g.quit = 1
##

idata = [
    ('player',os.path.join('data', 'images', 'player', 'greenguy.png'),(4,4,24,24)),
    ('enemy',os.path.join('data', 'images', 'player', 'kingstand.png'),(4,4,24,24)),
    ('shot',os.path.join('data', 'images', 'player', 'kingsitting.png'),(1,2,6,4)),
    ]

##The codes data has been updated to include information about the appropriate
##movement handlers for enemies.
##::

#the 5: here is used to tell PGU which sprite on the tileset is assigned to a function. Not sure what the dictionary is used for yet.
cdata = {
    1:(player_new,None),
    40:(enemy_new,{'move':enemy_move_line}),
    3:(enemy_new,{'move':enemy_move_sine}),
    #1:(enemy_new,{'move':enemy_move_circle}),
    }
##

#tiledata works like this:
#0x01 is the second tile (going from left to right) add a digit for each tile in the row. Columns are ignored. PGU just thinks of the tile sheet as one long row.

#the first entry in the tuple represents g.images['player']. This links the tile collision to a certain 
#sprite (in this case, the player sprite). The next entry in the tuple is cobble_road_block. This links the 
#player sprite to collide with the cobble_road_block sprite. The third entry in the tuple is a dictionary.
#The dictionary has the keys top, bottom, left, and right. The key's value (1 or 0) tells PGU if that edge of the tile can be collided with. If the value is one, that side is collided with. If the value is 0, it is ignored and there is no collision.
#tiledata = {
    #0x01:('player',cobble_road_block,{'top':1,'bottom':0,'left':1,'right':1}),
    #0x02:('player',cobble_road_block,{'top':1,'bottom':0,'left':1,'right':1}),
    #0x20:('player',tile_coin,None),
    #0x30:('player',tile_fire,None),
    #}
tiledata = castletiles.Tileset.tiledata

    
def init():
	GAME_DATA = load_game()
	#print load_game#GAME_DATA
	
	print GAME_DATA, SAVE_GAME_KEYS
	if has_valid_save_game() == False:
		print "Invalid game data. Restoring defaults."
		save_game(SAVE_GAME_KEYS)

	g = tilevid.Tilevid()
	g.hud = hud.HUD(g)
	
	##In init(), set the g.view size so that all the handlers will work properly.  (The player_loop one depends on view having the correct size.)
	##::
	g.view.w,g.view.h = SW,SH
	##
	
	g.screen = pygame.display.set_mode((SW,SH),SWSURFACE)
	g.frame = 0
	
	g.tga_load_tiles(os.path.join(DATA_PATH, 'castle.png'),(TW,TH),tiledata)
	##In init() I no longer have tga_load_level load the background layer, as 
	##we will generate our own multi-layered starfield.
	##::

	level_number = str(GAME_DATA['level'])

	g.tga_load_level(os.path.join(DATA_PATH, 'level' + level_number + '.tga'))
	##
	g.bounds = pygame.Rect(TW,TH,(len(g.tlayer[0])-2)*TW,(len(g.tlayer)-2)*TH)
	g.load_images(idata)
	g.run_codes(cdata,(0,0,25,17))
	pygame.font.init()
	g.font = pygame.font.SysFont('helvetica',16)
	
	return g
	
def run(g): 
	g.quit = 0
	##In run(), adding a pause variable to the game.
	##::
	g.pause = 0
	g.paused = 0
	g.TURN = True
	
	##
	
	t = timer.Timer(FPS)
	
	##In run(), initializing the stars.
	##::
	stars = []
	NS = 256
	for n in range(0,NS):
		stars.append([random.randrange(0,SW),random.randrange(0,SH),random.randrange(2,8)])
		
	##
	while not g.quit:
		for e in pygame.event.get():
			if e.type is QUIT: 
				pygame.quit()
				sys.exit()
			if e.type is KEYDOWN and e.key == K_ESCAPE: 
				g.quit = 1
			##In run(), in the event loop, checking for F10 for full screen, RETURN for pause.
			##::
			if e.type is KEYDOWN and e.key == K_F10:
				#g.screen = pygame.display.set_mode((SW,SH),FULLSCREEN|HWSURFACE|DOUBLEBUF)
				pygame.display.toggle_fullscreen()
				
			if e.type is KEYDOWN and e.key == K_RETURN:
				g.pause ^= 1
				g.paused = 1
				g.hud.pause()
		##
	
		##In run(), handles pause, and also renders the star field before the
		##foreground is painted.
		##::
		if not g.pause:
			#g.view.x += SPEED
			g.run_codes(cdata,(g.view.right/TW,0,1,17))
			
			g.loop()
		
			n = 0
			#for n in range(0,NS):
				#x,y,s = stars[n]
				#if ((g.frame*s)%8) < s:
					#x -= 1
				#if x < 0: x += SW
				#stars[n][0] = x
				#g.screen.set_at((x,y),(255,255,255))

			g.screen.fill((0,0,0))
			
			#g.hud.show_dialog("What's up? You paused the game.")
			g.hud.show_pending_dialog()
			g.paint(g.screen)
			g.hud.paint(g.screen)
			#img = g.font.render('%05d'%g.player.score,1,(0,0,0))
			#g.screen.blit(img,(0+1,SH-img.get_height()+1))
			#img = g.font.render('%05d'%g.player.score,1,(255,255,255))
			#g.screen.blit(img,(0,SH-img.get_height()))
			pygame.display.flip()
			
			g.frame += 1
	##

		t.tick()

def main():
	run(init())
